/*
    SPDX-FileCopyrightText: 2021 Volker Krause <vkrause@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#ifndef KOSMINDOORMAP_MAPCSSTYPES_H
#define KOSMINDOORMAP_MAPCSSTYPES_H

#include <osm/stringpool.h>

namespace KOSMIndoorMap {

class ClassSelectorKey : public OSM::StringKey {};
class LayerSelectorKey : public OSM::StringKey {};

}

#endif // KOSMINDOORMAP_MAPCSSTYPES_H
