
#ifndef KOSMINDOORROUTING_EXPORT_H
#define KOSMINDOORROUTING_EXPORT_H

#ifdef KOSMINDOORROUTING_STATIC_DEFINE
#  define KOSMINDOORROUTING_EXPORT
#  define KOSMINDOORROUTING_NO_EXPORT
#else
#  ifndef KOSMINDOORROUTING_EXPORT
#    ifdef KOSMIndoorRouting_EXPORTS
        /* We are building this library */
#      define KOSMINDOORROUTING_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KOSMINDOORROUTING_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KOSMINDOORROUTING_NO_EXPORT
#    define KOSMINDOORROUTING_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KOSMINDOORROUTING_DEPRECATED
#  define KOSMINDOORROUTING_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KOSMINDOORROUTING_DEPRECATED_EXPORT
#  define KOSMINDOORROUTING_DEPRECATED_EXPORT KOSMINDOORROUTING_EXPORT KOSMINDOORROUTING_DEPRECATED
#endif

#ifndef KOSMINDOORROUTING_DEPRECATED_NO_EXPORT
#  define KOSMINDOORROUTING_DEPRECATED_NO_EXPORT KOSMINDOORROUTING_NO_EXPORT KOSMINDOORROUTING_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KOSMINDOORROUTING_NO_DEPRECATED
#    define KOSMINDOORROUTING_NO_DEPRECATED
#  endif
#endif

#endif /* KOSMINDOORROUTING_EXPORT_H */
